# Copyright (c) 2011 Peter Marschall <peter@adpm.de>. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Description: A FilterList is a list of filters.  FilterLists are
# consumed by the matchedValues request control.  This module is
# pulled from the the end of Net::LDAP::Control::MatchedValues to 
# make the function a little less opaque. <whm@stanford.edu>
#
# This bit of code is a duplicate of the net::LDAP::FilterList package
# that is hidden at the bottom of the Net::LDAP::Filter module.  This
# is to workaround a problem with setting the matchet values request
# control.  Once Net::LDAPapi has support for this control then this
# package can be retired.  27-Aug-2012:whm

package Stanford::LDAP::FilterList;

use vars qw(@ISA $VERSION);
use Net::LDAP::Filter;

@ISA = qw(Net::LDAP::Filter);
$VERSION = "0.02";

my $Attr  = '[-;.:\d\w]*[-;\d\w]';
my $ErrStr;

sub new {
    my $self = shift;
    my $class = ref($self) || $self;

    my $me = bless [], $class;

    if (@_) {
        $me->parse(shift) or
            return undef;
    }
    $me;
}

sub errstr {
    return $ErrStr;
}

sub parse {
    my $self = shift;
    my $filterlist = shift;

    my @parsed = ();

    undef $ErrStr;

    # a filterlist is required
    if (!defined $filterlist) {
        $ErrStr = "Undefined filterlist";
        return undef;
    }

    # remove surrounding braces ((..)(..)(..)) -> (..)(..)(..)
    $filterlist =~s/^\((\(.*)\)$/$1/;

    while (length($filterlist)) {
        # process (attr op string)
        if ($filterlist =~ s/^\(\s*
                            ($Attr)\s*
                            ([:~<>]?=)
                            ((?:\\.|[^\\()]+)*)
                            \)\s*
                           //xo) {
            my $item = Net::LDAP::Filter::_encode($1,$2,$3);
            return undef  if (!$item);
            push(@parsed, $item);
            next;
        }

        # If we get here then there is an error in the filter string
        # so exit loop with data in $filterlist
        last;
    }

    if (length $filterlist) {
        # If we have anything left in the filter, then there is a problem
        $ErrStr = "Bad filterlist, error before " . substr($filterlist,0,20);
        return undef;
    }

    @$self = @parsed;
    
    $self;
}

sub as_string {
    my $l = shift;
    return '(' . join('', map { Net::LDAP::Filter::_string(%{$_}) } @{$l}) . ')';
}

1;
